/*
 * SPDX-FileCopyrightText: 2024 Slava Aseev <nullptrnine@basealt.ru>
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

import QtQuick 2.15
import QtQuick.Layouts 1.15

import org.kde.plasma.plasmoid 2.0
import org.kde.plasma.core 2.0 as PlasmaCore
import org.kde.plasma.extras 2.0 as PlasmaExtras
import org.kde.plasma.components 3.0 as PlasmaComponents
import org.kde.kirigami 2.11 as Kirigami
import org.kde.plasma.alt.private.weather 1.0


ColumnLayout {
    Layout.fillWidth: true
    Layout.fillHeight: true

    readonly property string coordinates: Plasmoid.configuration.coordinates
    readonly property string settlement: main.settlement
    readonly property string settlementLocale: Plasmoid.configuration.currentLocale
    readonly property string altValidationServer: Plasmoid.configuration.altValidationServer
    readonly property string weatherResource: Plasmoid.configuration.weatherResource

    spacing: PlasmaCore.Units.mediumSpacing

    property var detailsModel: [
        ["ic_wind", main.factModel.windSpeed],
        ["ic_pressure", main.factModel.pressure],
        ["ic_humidity", main.factModel.humidity]
    ]

    GeocodeListModel {
        id: geocodeListModel

        onLocationSearchDone: {
            Plasmoid.configuration.geocodeSettlement = geocodeListModel.getInfo(0);
            Plasmoid.configuration.currentLocale = Qt.locale().name;
        }
    }

    RowLayout {
        Layout.fillWidth: true

        PlasmaCore.IconItem {
            Layout.fillHeight: true
            readonly property int preferredIconSize: PlasmaCore.Units.iconSizes.smallMedium

            Layout.alignment: Qt.AlignTop
            Layout.preferredHeight: preferredIconSize
            Layout.preferredWidth: preferredIconSize

            source: "mark-location"
        }

        ColumnLayout {
            Item {
                id: settlementItem
                Layout.fillWidth: true
                height: dummyText.height

                PlasmaExtras.Heading {
                    id: dummyText

                    visible: false
                    type: PlasmaExtras.Heading.Type.Primary
                    level: 1
                    text: "AAA"
                }

                PlasmaExtras.Heading {
                    anchors.left: parent.left
                    width: settlementItem.width

                    level: 1
                    type: PlasmaExtras.Heading.Type.Primary
                    text: settlement
                    elide: Text.ElideRight
                    wrapMode: Text.NoWrap
                }
            }

            /* XXX: Workaround for Education */
            spacing: PlasmaCore.Units.smallSpacing < 4 ? 4 : 0

            PlasmaExtras.Heading {
                Layout.alignment: Qt.AlignTop
                verticalAlignment: Qt.AlignTop

                level: 5
                type: PlasmaExtras.Heading.Type.Secondary
                text: main.currentDate ? main.currentDate.toLocaleDateString() : ""
            }
        }
    }

    Item {
        Layout.fillHeight: true
    }

    RowLayout {
        id: factTempLayout

        /* XXX: Workaround for Education */
        spacing: PlasmaCore.Units.largeSpacing < 18 ? 24 : PlasmaCore.Units.mediumSpacing

        Item {
            Layout.fillWidth: true
        }

        PlasmaCore.IconItem {
            id: factIcon

            readonly property int preferredIconSize: PlasmaCore.Units.iconSizes.huge * 1.6

            Layout.minimumWidth: preferredIconSize
            Layout.minimumHeight: preferredIconSize

            source: main.factModel.icon
            roundToIconSize: false

            PlasmaCore.ToolTipArea {
                id: iconToolTip
                anchors.fill: parent
                mainText: main.getWeatherDescription(main.factModel.condition)
            }
        }

        Item {
            width: PlasmaCore.Units.gridUnit * 8
            height: tempLabel.implicitHeight + feelsLikeText.implicitHeight

            PlasmaComponents.Label {
                anchors.horizontalCenter: parent.horizontalCenter
                id: tempLabel

                font.pointSize: Kirigami.Theme.defaultFont.pointSize * 5
                text: main.factModel.temp
            }

            PlasmaExtras.Heading {
                id: feelsLikeText
                anchors.verticalCenter: tempLabel.bottom
                anchors.horizontalCenter: tempLabel.horizontalCenter
                level: 3
                type: PlasmaExtras.Heading.Type.Secondary
                text: i18nc("@label", "Feels like %1", main.factModel.feelsLike)
            }
        }

        Item {
            Layout.fillWidth: true
        }

        Column {
            spacing: PlasmaCore.Units.smallSpacing
            visible: main.isInSystemTray
            Layout.fillWidth: main.isInSystemTray

            WindPressureHumidity {
                model: detailsModel
            }
        }
    }

    Row {
        spacing: PlasmaCore.Units.largeSpacing
        visible: !main.isInSystemTray
        anchors.horizontalCenter: parent.horizontalCenter

        WindPressureHumidity {
            model: detailsModel
        }
    }

    Item {
        Layout.fillHeight: !main.isInSystemTray
    }

    Component.onCompleted: {
        if (Qt.locale().name != settlementLocale && coordinates) {
            geocodeListModel.clear();
            geocodeListModel.searchGeocodes(altValidationServer, weatherResource, coordinates);
        }
    }
}
