/*
 * SPDX-FileCopyrightText: 2024 Slava Aseev <nullptrnine@basealt.ru>
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

import QtQuick 2.15
import QtQuick.Controls 2.15 as QQC2
import QtQuick.Layouts 1.15

import org.kde.kirigami 2.15 as Kirigami
import org.kde.plasma.plasmoid 2.0
import org.kde.plasma.alt.private.weather 1.0


ColumnLayout {
    id: weatherStationConfigPage

    property string cfg_altValidationServer
    property string cfg_weatherResource
    property alias cfg_forecastColumnsCount: columnsCountSpin.value
    property string cfg_coordinates
    property string cfg_geocodeSettlement
    property string cfg_currentLocale
    property string cfg_shortNameOfMonth
    property bool cfg_autoLocation


    readonly property var coordinates: cfg_coordinates
    readonly property var geocodeSettlement: cfg_geocodeSettlement
    readonly property var currentLocale: cfg_currentLocale
    readonly property var forecastColumnsCount: cfg_forecastColumnsCount
    readonly property bool haveLocation: !!coordinates
    readonly property bool canSearch: !!searchStringEdit.text

    // The model property `isValidatingInput` doesn't account for the timer delay
    // We use a custom property to provide a more responsive feedback
    property bool isSearching: false

    GeocodeListModel {
        id: geocodeListModel
        onLocationSearchDone: {
            isSearching = false;
        }
    }

    Kirigami.FormLayout {
        id: formLayout

        QQC2.SpinBox {
            id: columnsCountSpin

            Kirigami.FormData.label: i18nc("@label:spinbox", "Forecast width:")

            textFromValue: function(value) {
                return (i18np("columns count", "%1 columns", value));
            }
            valueFromText: function(text) {
                return parseInt(text);
            }

            from: 3
            to: 10
            editable: true
        }

        QQC2.CheckBox {
            id: autoLocationCheckBox

            property var tmpCoordinates: null
            property var tmpGeocodeSettlement: null

            Kirigami.FormData.label: i18nc("@option:check", "Set location automatically:")
            checked: cfg_autoLocation
            onToggled: {
                cfg_autoLocation = checked

                if (checked) {
                    tmpCoordinates = cfg_coordinates;
                    tmpGeocodeSettlement = cfg_geocodeSettlement;
                    cfg_coordinates = null;
                    cfg_geocodeSettlement = null;
                }
                else {
                    cfg_coordinates = tmpCoordinates;
                    cfg_geocodeSettlement = tmpGeocodeSettlement;
                }
            }
        }

        QQC2.Label {
            Kirigami.FormData.label: i18nc("@label", "Location:")

            Layout.fillWidth: true
            elide: Text.ElideRight
            opacity: haveLocation ? 1 : 0.7

            function geocodeInfo() {
                if (haveLocation) {
                    return geocodeSettlement;
                }
                return i18nc("No location is currently selected", "None selected");
            }

            text: geocodeInfo()
        }

        Item {
            // This dummy item adds some spacing and makes the form layout less jumpy
            Kirigami.FormData.isSection: true
            implicitWidth: Math.min(Kirigami.Units.gridUnit * 15, weatherStationConfigPage.width - Kirigami.Units.gridUnit)
        }
    }

    Kirigami.SearchField {
        id: searchStringEdit

        Layout.fillWidth: true

        focus: true
        enabled: !autoLocationCheckBox.checked
        placeholderText: i18nc("@info:placeholder", "Enter the name of the locality or coordinates in the format: 'latitude, longitude'")
        validator: RegExpValidator{ regExp: /^[0-9а-яА-Яa-zA-ZіІїЇєЄәӘңүҮұҰқҚөӨһӘ̃ә̃ÇçĞğİıÖöŞşÜü.,\[\]\';`\- ]+$/ }

        Timer {
            id: searchDelayTimer
            interval: 500
            onTriggered: {
                if (!canSearch) {
                    geocodeListModel.clear();
                    return;
                }
                geocodeListModel.searchGeocodes(cfg_altValidationServer, cfg_weatherResource, searchStringEdit.text);
            }
        }

        onTextChanged: {
            isSearching = text.length > 0
            searchDelayTimer.restart();
        }

        Keys.onUpPressed: {
            if (locationListView.currentIndex != 0) {
                locationListView.currentIndex--;
            }
            event.accepted = true;
        }
        Keys.onDownPressed: {
            if (locationListView.currentIndex != locationListView.count - 1) {
                locationListView.currentIndex++;
            }
            event.accepted = true;
        }
        Keys.onEscapePressed: {
            if (searchStringEdit.text.length === 0) {
                event.accepted = false;
                return;
            }
            searchStringEdit.clear();
            event.accepted = true;
        }
    }

    QQC2.ScrollView {
        Layout.fillWidth: true
        Layout.fillHeight: true
        Layout.maximumHeight: weatherStationConfigPage.height - formLayout.height - searchStringEdit.height - 3 * Kirigami.Units.smallSpacing

        enabled: !autoLocationCheckBox.checked

        Component.onCompleted: {
            background.visible = true;
        }

        //enabled: true //Object.keys(providers).length > 0

        ListView {
            id: locationListView
            model: geocodeListModel
            focus: true
            activeFocusOnTab: true
            keyNavigationEnabled: true
            enabled: !autoLocationCheckBox.checked

            onCurrentIndexChanged: {
                const coords = geocodeListModel.getLatLon(locationListView.currentIndex);
                var settlement = geocodeListModel.getName(locationListView.currentIndex);
                const description = geocodeListModel.getDescription(locationListView.currentIndex);

                if (description) {
                    settlement += ", " + description;
                }

                weatherStationConfigPage.cfg_coordinates = coords[0] + " " + coords[1];
                weatherStationConfigPage.cfg_geocodeSettlement = settlement;
                weatherStationConfigPage.cfg_currentLocale = Qt.locale().name;
            }

            delegate: QQC2.ItemDelegate {
                width: locationListView.width
                text: model.display
                highlighted: ListView.isCurrentItem

                onClicked: {
                    locationListView.forceActiveFocus();
                    locationListView.currentIndex = index;
                }
            }

            // To avoid start with a highlighted item on the next search
            onCountChanged: {
                if (count === 0) {
                    currentIndex = -1;
                }
            }

            Keys.forwardTo: searchStringEdit

            Kirigami.PlaceholderMessage {
                id: listViewPlaceholder
                anchors.centerIn: parent
                width: parent.width - Kirigami.Units.gridUnit
                visible: locationListView.count === 0 && !isSearching

                function getErrorDescription(statusCode) {
                    const descriptions = {
                        400: i18nc("Error description", "%1 Bad request", statusCode),
                        401: i18nc("Error description", "%1 Unauthorized", statusCode),
                        429: i18nc("Error description", "%1 Limit exceed", statusCode),
                        500: i18nc("Error description", "%1 Internal server error", statusCode),
                    };

                    const description = descriptions[statusCode];
                    if (!description) {
                        return i18nc("Error description", "Error %1", statusCode);
                    }
                    return description;
                }

                text: {
                    if (canSearch) {
                        if (geocodeListModel.statusCode != 404) {
                            return i18nc("@info", "Search failed: %1", getErrorDescription(geocodeListModel.statusCode));
                        } else {
                            return i18nc("@info", "No data found for '%1'", searchStringEdit.text);
                        }
                    } else if (haveLocation) {
                        return i18nc("@info", "Search for a settlement to change your location");
                    } else {
                        return i18nc("@info", "Search for a settlement to set your location");
                    }
                }
            }

            QQC2.BusyIndicator {
                id: busy
                anchors.centerIn: parent
                visible: locationListView.count === 0 && isSearching
            }
        }
    }
}
