/*
 * SPDX-FileCopyrightText: 2024 Slava Aseev <nullptrnine@basealt.ru>
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

import QtQuick 2.0
import QtQuick.Layouts 1.1

import org.kde.plasma.plasmoid 2.0
import org.kde.plasma.core 2.0 as PlasmaCore
import org.kde.plasma.extras 2.0 as PlasmaExtras
import org.kde.plasma.alt.private.weather 1.0
import org.kde.plasma.components 3.0 as PlasmaComponents

Item {
    id: main

    Plasmoid.backgroundHints: PlasmaCore.Types.DefaultBackground | PlasmaCore.Types.ConfigurableBackground

    readonly property string weatherResource: Plasmoid.configuration.weatherResource
    readonly property string validationServer: Plasmoid.configuration.altValidationServer
    readonly property string shortNameOfMonth: Plasmoid.configuration.shortNameOfMonth
    readonly property string coordinates: Plasmoid.configuration.coordinates

    readonly property var weatherSrc: {
        /* TODO: do not request for the new data if forecastColumnsCount reduces */
        return "weather|" +
            validationServer + '|' +
            weatherResource + '|' +
            Qt.locale().name + "|" +
            coordinates + "|" +
            (Plasmoid.configuration.forecastColumnsCount + 1);
    }

    readonly property string longitude: coordinates ? coordinates.split(' ')[0] : null
    readonly property string latitude: coordinates ? coordinates.split(' ')[1] : null
    readonly property bool haveWeatherData: !!dataSource.currentData && dataSource.currentData.type == "ok"

    readonly property string currentLon: longitude ? longitude : factModel.lon
    readonly property string currentLat: latitude ? latitude : factModel.lat

    property int status: Util.NeedToConfigure
    property var lastUpdateTime: timeSource.data["Local"].DateTime

    readonly property string settlement: {
        if (Plasmoid.configuration.geocodeSettlement) {
            return Plasmoid.configuration.geocodeSettlement;
        }
        return main.factModel.settlement;
    }

    //readonly property var statusNames: ["normal", "need_to_configure", "connecting", "timeout", "error", "emergency"]
    //onStatusChanged: console.warn("[status] changed to:", statusNames[status])

    readonly property bool isInSystemTray: {
        return (Plasmoid.containmentType & PlasmaCore.Types.CustomEmbeddedContainment) |
               (Plasmoid.containmentDisplayHints & PlasmaCore.Types.ContainmentForcesSquarePlasmoids);
    }

    function dumpWeatherData() {
        if (dataSource.currentData) {
            console.warn("---> [Weather Data]:", JSON.stringify(dataSource.currentData, null, 2));
        }
    }

    function iconName(name) {
        return "ya_weather_" + name;
    }

    function getLogoLang() {
        return Qt.locale().name.startsWith("ru") ? "ru" : "en";
    }

    readonly property string yandexLogoPath: {
        return Util.getResourcesDir() + "ya_logo/logo_weather_" + getLogoLang() + "_" + main.getThemeType();
    }

    function isDark(color, threshold) {
        const gray = (color.r * 11 + color.g * 16 + color.b * 5) / 32;
        return gray < threshold;
    }

    function getThemeType() {
        return isDark(PlasmaCore.Theme.textColor, 0.65) ? "light_bk" : "dark_bk";
    }

    function getWeatherDescription(condition) {
        const descriptions = {
            "clear": i18nc("Weather description", "Clear"),
            "partly-cloudy": i18nc("Weather description", "Partly cloudy"),
            "cloudy": i18nc("Weather description", "Cloudy"),
            "overcast": i18nc("Weather description", "Overcast"),
            "drizzle": i18nc("Weather description", "Drizzle"),
            "light-rain": i18nc("Weather description", "Light rain"),
            "rain": i18nc("Weather description", "Rain"),
            "moderate-rain": i18nc("Weather description", "Moderate rain"),
            "heavy-rain": i18nc("Weather description", "Heavy rain"),
            "continuous-heavy-rain": i18nc("Weather description", "Continuous heavy rain"),
            "showers": i18nc("Weather description", "Showers"),
            "wet-snow": i18nc("Weather description", "Sleet"),
            "light-snow": i18nc("Weather description", "Light snow"),
            "snow": i18nc("Weather description", "Snow"),
            "snow-showers": i18nc("Weather description", "Snowfall"),
            "hail": i18nc("Weather description", "Hail"),
            "thunderstorm": i18nc("Weather description", "Thunderstorm"),
            "thunderstorm-with-rain": i18nc("Weather description", "Rain, thunderstorm"),
            "thunderstorm-with-hail": i18nc("Weather description", "Thunderstorm, hail")
        };

        const description = descriptions[condition];
        if (!description) {
            return i18nc("Weather description", "Unknown weather description");
        }

        return description;
    }

    function getErrorDescription(statusCode) {
        const descriptions = {
            400: i18nc("Error description", "%1 Bad request", statusCode),
            401: i18nc("Error description", "%1 Unauthorized", statusCode),
            429: i18nc("Error description", "%1 Limit exceed", statusCode),
            500: i18nc("Error description", "%1 Internal server error", statusCode),
        };

        const description = descriptions[statusCode];
        if (!description) {
            return i18nc("Error description", "%1 Error", statusCode);
        }
        return description;
    }

    function getErrorText() {
        var res = i18n("Weather information retrieval failed");
        if (main.status === Util.Timeout || (main.errorModel && main.errorModel.type == "unknown_error")) {
            res += ":\n" + i18n("timed out.");
        }
        else if (main.errorModel) {
            res += ":\n" + main.getErrorDescription(main.errorModel.status_code);
        }
        return res;
    }

    function prettifyTemp(temp) {
        var result = temp;
        if (temp > 0) {
            result = "+" + temp.toString();
        }
        result += "°";
        return result;
    }

    function prettifyWindSpeed(speed) {
        if (speed % 1 == 0) {
            speed += ".0";
        }
        return i18nc("Wind speed", "%1 m/s", speed);
    }

    function prettifyPressure(pressure) {
        return i18nc("Pressure", "%1 mm", pressure);
    }

    function prettifyHumidity(humidity) {
        return humidity + "%";
    }

    function windDirIconAngle(windDir) {
        return {
            n: -180,
            ne: -135,
            e: -90,
            se: -45,
            s: 0,
            sw: 45,
            w: 90,
            nw: 135
        }[windDir];
    }

    function windDirPrettify(windDir) {
        return {
            n: i18nc("Wind direction", "N"),
            ne: i18nc("Wind direction", "NE"),
            e: i18nc("Wind direction", "E"),
            se: i18nc("Wind direction", "SE"),
            s: i18nc("Wind direction", "S"),
            sw: i18nc("Wind direction", "SW"),
            w: i18nc("Wind direction", "W"),
            nw: i18nc("Wind direction", "NW"),
            c: i18nc("Wind direction", "Calm"),
        }[windDir];
    }

    function isCurrentDate(dateString) {
        if (!currentDate) {
            return false;
        }

        const date = Date.fromLocaleDateString(Qt.locale(), dateString, "yyyy-MM-dd");
        const today = new Date(currentDate.getTime());
        today.setHours(0, 0, 0, 0);
        return today.getTime() === date.getTime();
    }

    function getDayOfWeek(dateString) {
        const date = Date.fromLocaleDateString(Qt.locale(), dateString, "yyyy-MM-dd");
        const day = date.toLocaleDateString(Qt.locale(), "ddd");
        return day.charAt(0).toUpperCase() + day.slice(1);
    }

    function getDayOfMonths(dateString) {
        const date = Date.fromLocaleDateString(Qt.locale(), dateString, "yyyy-MM-dd");
        return date.toLocaleDateString(Qt.locale(), "dd " + (shortNameOfMonth ? "MMM" : "MMMM"))
    }

    readonly property var forecastWeekModel: {
        const model = [];
        const data = dataSource.currentData;

        if (!data || data.type != "ok") {
            return model
        }
        const weather = data.body.weather;

        var daysCount = Math.min(weather.forecasts.length, Plasmoid.configuration.forecastColumnsCount);
        /* 9 days max because night temp is taken from the next day */
        if (daysCount > 9) {
            daysCount = 9;
        }

        for (var i = 0; i < daysCount; ++i) {
            const forecast = weather.forecasts[i];
            const nextForecast = weather.forecasts[i + 1];
            const forecastParts = forecast.parts;
            const nextForecastParts = nextForecast.parts;

            const forecastInfo = {
                dayTemp: prettifyTemp(forecastParts.day_short.temp),
                nightTemp: prettifyTemp(nextForecastParts.night_short.temp),
                icon: iconName(forecastParts.day_short.icon),
                dayOfWeek: getDayOfWeek(forecast.date),
                dayOfMonth: getDayOfMonths(forecast.date),
                condition: forecastParts.day_short.condition,
                currentDate: isCurrentDate(forecast.date)
            };
            model.push(forecastInfo);
        }

        return model;
    }

    readonly property var forecastHourModel: {
        const model = [];
        if (!currentDate) {
            return model;
        }

        const data = dataSource.currentData;

        if (!data || data.type != "ok") {
            return model;
        }
        const weather = data.body.weather;

        const columnsCount = Plasmoid.configuration.forecastColumnsCount;

        var dayIndex = 0;
        var hour = currentDate.getHours();

        for (var i = 0; i < columnsCount; ++i) {
            if (hour % 24 == 0 && i != 0) {
                hour = 0;
                dayIndex++;
            }

            const forecast = weather.forecasts[dayIndex].hours[hour];

            model.push({
                temp: prettifyTemp(forecast.temp),
                icon: iconName(forecast.icon),
                hour: hour + ":00",
                condition: forecast.condition,
            });

            hour++;
        }

        return model;
    }

    readonly property var factModel: {
        const data = dataSource.currentData;

        if (!data || data.type != "ok") {
            return {
                temp: "",
                feelsLike: "",
                icon: "",
                windSpeed: "",
                windDir: "",
                pressure: "",
                humidity: "",
                condition: "",
                settlement: "",
                lat: "",
                lon: "",
            };
        }
        const fact = data.body.weather.fact;
        const geo = data.body.weather.geo_object;
        const info = data.body.weather.info;

        return {
            temp: prettifyTemp(fact.temp),
            feelsLike: prettifyTemp(fact.feels_like),
            icon: iconName(fact.icon),
            windSpeed: prettifyWindSpeed(fact.wind_speed),
            windDir: fact.wind_dir,
            pressure: prettifyPressure(fact.pressure_mm),
            humidity: prettifyHumidity(fact.humidity),
            condition: fact.condition,
            settlement: geo.locality.name,
            lat: info.lat,
            lon: info.lon,
        };
    }

    readonly property var errorModel: {
        const data = dataSource.currentData;

        if (!data) {
            return null;
        }

        if (data.type != "ok") {
            return data;
        }

        return null;
    }

    PlasmaCore.DataSource {
        id: dataSource

        readonly property var newWeatherData: {
            const data = dataSource.data[weatherSrc];
            if (!data || !data.weather) {
                return null;
            }

            return data.weather;
        }

        readonly property var currentData: {
            const data = newWeatherData;
            if (!data) {
                return null;
            }

            if (data.type == "ok") {
                status = Util.Normal;
                lastUpdateTime = timeSource.data["Local"].DateTime
                return data;
            }
            else {
                /* Show old data only while the day is the same */
                if (currentData && currentData.type == "ok" && lastUpdateTime.getDay() == timeSource.data["Local"].DateTime.getDay()) {
                    status = Util.Emergency;
                    return currentData;
                }
                else {
                    status = Util.Error;
                    return data;
                }
            }
        }

        engine: "altweather"
        connectedSources: weatherSrc
        intervalAlignment: PlasmaCore.Types.NoAlignment
        interval: Plasmoid.configuration.updateInterval * 60 * 1000

        onConnectedSourcesChanged: {
            if (connectedSources) {
                if (haveWeatherData) {
                    status = Util.Normal;
                    return;
                }
                status = Util.Connecting;
                connectionTimeoutTimer.start();
            }
        }

        onCurrentDataChanged: {
            if (currentData) {
                connectionTimeoutTimer.stop();
            }
        }
    }

    PlasmaCore.DataSource {
        id: timeSource
        engine: "time"
        connectedSources: "Local"
        interval: 60000
        intervalAlignment: PlasmaCore.Types.AlignToMinute
    }

    readonly property var currentDate: {
        if (!dataSource.currentData) {
            return;
        }

        var now = timeSource.data["Local"].DateTime;
        var utc = now.getTime() + now.getTimezoneOffset() * 60000;
        var settlementLocal = utc + dataSource.currentData.body.weather.info.tzinfo.offset * 1000;
        return new Date(settlementLocal);
    }

    Timer {
        id: connectionTimeoutTimer

        interval: 30 * 1000
        repeat: false
        onTriggered: {
            status = Util.Timeout;
        }
    }

    Plasmoid.fullRepresentation: CombinedView {}
    Plasmoid.compactRepresentation: TrayView {}

    Plasmoid.toolTipItem: ColumnLayout {
        spacing: 0

        PlasmaExtras.Heading {
            level: 3
            elide: Text.ElideRight
            text: {
                if (status === Util.Timeout || main.status === Util.Error) {
                    return i18n("Error!");
                }
                if (status === Util.NeedToConfigure) {
                    return i18nc("@info:tooltip %1 is the plasmoid name", "Click to configure %1", Plasmoid.title)
                }
                return settlement;
            }
        }

        Repeater {
            model: toolTipModel
            PlasmaComponents.Label {
                opacity: 0.6
                text: modelData
            }
        }
    }

    function optLabel(prefix, msg) {
        const pref = Plasmoid.configuration.showLabelsInToolTip ? prefix : "";
        return pref + " " + msg;
    }

    readonly property var toolTipModel: {
        if (main.status === Util.Timeout || main.status === Util.Error) {
            return [getErrorText()];
        }
        else if (main.status === Util.Connecting) {
            return [];
        }

        if (!settlement) {
            return [];
        }

        const tooltips = [];

        if (Plasmoid.configuration.showCondTempInToolTip) {
            tooltips.push(getWeatherDescription(factModel.condition) + ", " + factModel.temp);
        }
        if (Plasmoid.configuration.showWindInToolTip) {
            var windStr = main.factModel.windDir != 'c' ? (factModel.windSpeed + ", ") : "";
            windStr += windDirPrettify(factModel.windDir);
            tooltips.push(optLabel(i18nc("@label", "Wind:"), windStr));
        }
        if (Plasmoid.configuration.showPressureInToolTip) {
            tooltips.push(optLabel(i18nc("@label", "Pressure:"), factModel.pressure));
        }
        if (Plasmoid.configuration.showHumidityInToolTip) {
            tooltips.push(optLabel(i18nc("@label", "Humidity:"), factModel.humidity));
        }

        return tooltips;
    }
}
